@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 # -*- coding: utf-8; -*-
 
 """\
@@ -55,10 +55,23 @@ def main():
     args = parser.parse_args()
 
     try:
+        run_anywhere = "init clean list-models serial"
+
+        in_project_dir = os.path.isdir(e.src_dir)
+        if not in_project_dir and current_command not in run_anywhere:
+            raise Abort("No project found in this directory.")
+
         e.process_args(args)
 
-        if current_command not in 'clean init' and not os.path.isdir(e.build_dir):
-            os.makedirs(e.build_dir)
+        if current_command not in run_anywhere:
+            # For valid projects create .build & lib
+            if not os.path.isdir(e.build_dir):                
+                os.makedirs(e.build_dir)
+
+            if not os.path.isdir(e.lib_dir):
+                os.makedirs(e.lib_dir)
+                with open('lib/.holder', 'w') as f:
+                    f.write("")
 
         args.func(args)
     except Abort as exc:
